# Auto generated from information_resource_registry.yaml by pythongen.py version: 0.0.1
# Generation date: 2024-03-08T11:20:09
# Schema: Information-Resource-Registry-Schema
#
# id: https://w3id.org/biolink/information_resource_registry.yaml
# description:
# license: https://creativecommons.org/publicdomain/zero/1.0/

import dataclasses
import re
from jsonasobj2 import JsonObj, as_dict
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from linkml_runtime.linkml_model.meta import EnumDefinition, PermissibleValue, PvFormulaOptions

from linkml_runtime.utils.slot import Slot
from linkml_runtime.utils.metamodelcore import empty_list, empty_dict, bnode
from linkml_runtime.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
from linkml_runtime.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from linkml_runtime.utils.formatutils import camelcase, underscore, sfx
from linkml_runtime.utils.enumerations import EnumDefinitionImpl
from rdflib import Namespace, URIRef
from linkml_runtime.utils.curienamespace import CurieNamespace
from linkml_runtime.linkml_model.types import String

metamodel_version = "1.7.0"
version = "1.0.0"

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
BIOGRID = CurieNamespace('BIOGRID', 'http://identifiers.org/biogrid/')
SO = CurieNamespace('SO', 'http://purl.obolibrary.org/obo/SO_')
BIOLINK = CurieNamespace('biolink', 'https://w3id.org/biolink/')
INFORES = CurieNamespace('infores', 'https://w3id.org/biolink/infores/')
LINKML = CurieNamespace('linkml', 'https://w3id.org/linkml/')
OBOINOWL = CurieNamespace('oboInOwl', 'http://www.geneontology.org/formats/oboInOwl#')
RDF = CurieNamespace('rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#')
RDFS = CurieNamespace('rdfs', 'http://www.w3.org/2000/01/rdf-schema#')
SKOS = CurieNamespace('skos', 'http://www.w3.org/2004/02/skos/core#')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = INFORES


# Types

# Class references



@dataclass
class InformationResourceContainer(YAMLRoot):
    """
    A collection of information resources
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = INFORES["InformationResourceContainer"]
    class_class_curie: ClassVar[str] = "infores:InformationResourceContainer"
    class_name: ClassVar[str] = "InformationResourceContainer"
    class_model_uri: ClassVar[URIRef] = INFORES.InformationResourceContainer

    information_resources: Optional[Union[Union[dict, "InformationResource"], List[Union[dict, "InformationResource"]]]] = empty_list()

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if not isinstance(self.information_resources, list):
            self.information_resources = [self.information_resources] if self.information_resources is not None else []
        self.information_resources = [v if isinstance(v, InformationResource) else InformationResource(**as_dict(v)) for v in self.information_resources]

        super().__post_init__(**kwargs)


@dataclass
class InformationResource(YAMLRoot):
    """
    A database or knowledgebase and its supporting ecosystem of interfaces and services that deliver content to
    consumers (e.g. web portals, APIs, query endpoints, streaming services, data downloads, etc.). A single
    Information Resource by this definition may span many different datasets or databases, and include many access
    endpoints and user interfaces. Information Resources include project-specific resources such as a Translator
    Knowledge Provider, and community knowledgebases like ChemBL, OMIM, or DGIdb.
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = INFORES["InformationResource"]
    class_class_curie: ClassVar[str] = "infores:InformationResource"
    class_name: ClassVar[str] = "InformationResource"
    class_model_uri: ClassVar[URIRef] = INFORES.InformationResource

    id: str = None
    status: Optional[Union[str, "InformationResourceStatusEnum"]] = None
    name: Optional[str] = None
    xref: Optional[Union[str, List[str]]] = empty_list()
    synonym: Optional[Union[str, List[str]]] = empty_list()
    description: Optional[str] = None
    knowledge_level: Optional[Union[str, "KnowledgeLevelEnum"]] = None
    agent_type: Optional[Union[str, "AgentTypeEnum"]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self._is_empty(self.id):
            self.MissingRequiredField("id")
        if not isinstance(self.id, str):
            self.id = str(self.id)

        if self.status is not None and not isinstance(self.status, InformationResourceStatusEnum):
            self.status = InformationResourceStatusEnum(self.status)

        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if not isinstance(self.xref, list):
            self.xref = [self.xref] if self.xref is not None else []
        self.xref = [v if isinstance(v, str) else str(v) for v in self.xref]

        if not isinstance(self.synonym, list):
            self.synonym = [self.synonym] if self.synonym is not None else []
        self.synonym = [v if isinstance(v, str) else str(v) for v in self.synonym]

        if self.description is not None and not isinstance(self.description, str):
            self.description = str(self.description)

        if self.knowledge_level is not None and not isinstance(self.knowledge_level, KnowledgeLevelEnum):
            self.knowledge_level = KnowledgeLevelEnum(self.knowledge_level)

        if self.agent_type is not None and not isinstance(self.agent_type, AgentTypeEnum):
            self.agent_type = AgentTypeEnum(self.agent_type)

        super().__post_init__(**kwargs)


# Enumerations
class InformationResourceStatusEnum(EnumDefinitionImpl):
    """
    The status of the infores identifier
    """
    released = PermissibleValue(text="released")
    deprecated = PermissibleValue(text="deprecated")
    draft = PermissibleValue(text="draft")
    modified = PermissibleValue(text="modified")

    _defn = EnumDefinition(
        name="InformationResourceStatusEnum",
        description="The status of the infores identifier",
    )

class KnowledgeLevelEnum(EnumDefinitionImpl):
    """
    The level of knowledge that supports an edge or node. This is a general categorization of the type of evidence
    that supports a statement, and is not intended to be a comprehensive description of the evidence. For example, a
    statement may be supported by a single publication, but that publication may contain multiple types of evidence,
    such as a computational prediction and a manual curation. In this case, the knowledge level would be "curated",
    and the evidence would be described in more detail in the evidence graph.
    """
    curated = PermissibleValue(
        text="curated",
        description="""knowledge generated through manual curation  or interpretation of data or published study results""")
    predicted = PermissibleValue(
        text="predicted",
        description="""predictions generated computationally through inference over less direct forms of evidence (without human  intervention or review)""")
    text_mined = PermissibleValue(
        text="text_mined",
        description="knowledge extracted from published text by NLP agents (without human intervention or review)")
    correlation = PermissibleValue(
        text="correlation",
        description="""statistical correlations calculated between variables in a clinical or omics dataset, by an automated  analysis pipeline""")
    observed = PermissibleValue(
        text="observed",
        description="""edge reports a phenomenon that was reported/observed to have occurred (and possibly some quantification,  e.g. how many times, at what frequency)""")
    other = PermissibleValue(
        text="other",
        description="knowledge level may not fit into the categories above, or is not provided/known")
    mixed = PermissibleValue(
        text="mixed",
        description="""used for sources that might provide edges with different knowledge levels, e.g.correlations in addition to  curated Edges - set tag to Curated, unless predicate rules override""")

    _defn = EnumDefinition(
        name="KnowledgeLevelEnum",
        description="""The level of knowledge that supports an edge or node.  This is a general categorization of the type of evidence that supports a statement, and is not intended to be a comprehensive description of the evidence.  For example, a statement may be supported by a single publication, but that publication may contain multiple types of evidence, such as a computational prediction and a manual curation.  In this case, the knowledge level would be \"curated\", and the evidence would be described in more detail in the evidence graph.""",
    )

class AgentTypeEnum(EnumDefinitionImpl):
    """
    The type of agent that supports an edge or node. This is a general categorization of the type of agent that
    supports a statement, and is not intended to be a comprehensive description of the agent. For example, a statement
    may be supported by a single publication, but that publication may contain multiple types of evidence, such as a
    computational prediction and a manual curation. In this case, the agent type would be "publication", and the
    evidence would be described in more detail in the evidence graph.
    """
    not_provided = PermissibleValue(
        text="not_provided",
        description="agent type is not provided or known")
    computational_model = PermissibleValue(
        text="computational_model",
        description="a computational model, such as a machine learning model")

    _defn = EnumDefinition(
        name="AgentTypeEnum",
        description="""The type of agent that supports an edge or node.  This is a general categorization of the type of agent that supports a statement, and is not intended to be a comprehensive description of the agent.  For example, a statement may be supported by a single publication, but that publication may contain multiple types of evidence, such as a computational prediction and a manual curation.  In this case, the agent type would be \"publication\", and the evidence would be described in more detail in the evidence graph.""",
    )

# Slots
class slots:
    pass

slots.status = Slot(uri=INFORES.status, name="status", curie=INFORES.curie('status'),
                   model_uri=INFORES.status, domain=None, range=Optional[Union[str, "InformationResourceStatusEnum"]])

slots.information_resources = Slot(uri=INFORES.information_resources, name="information_resources", curie=INFORES.curie('information_resources'),
                   model_uri=INFORES.information_resources, domain=None, range=Optional[Union[Union[dict, InformationResource], List[Union[dict, InformationResource]]]])

slots.name = Slot(uri=RDFS.label, name="name", curie=RDFS.curie('label'),
                   model_uri=INFORES.name, domain=None, range=Optional[str])

slots.id = Slot(uri=INFORES.id, name="id", curie=INFORES.curie('id'),
                   model_uri=INFORES.id, domain=None, range=str)

slots.xref = Slot(uri=INFORES.xref, name="xref", curie=INFORES.curie('xref'),
                   model_uri=INFORES.xref, domain=None, range=Optional[Union[str, List[str]]])

slots.synonym = Slot(uri=INFORES.synonym, name="synonym", curie=INFORES.curie('synonym'),
                   model_uri=INFORES.synonym, domain=None, range=Optional[Union[str, List[str]]])

slots.description = Slot(uri=RDFS.comment, name="description", curie=RDFS.curie('comment'),
                   model_uri=INFORES.description, domain=None, range=Optional[str])

slots.knowledge_level = Slot(uri=INFORES.knowledge_level, name="knowledge level", curie=INFORES.curie('knowledge_level'),
                   model_uri=INFORES.knowledge_level, domain=None, range=Optional[Union[str, "KnowledgeLevelEnum"]])

slots.agent_type = Slot(uri=INFORES.agent_type, name="agent type", curie=INFORES.curie('agent_type'),
                   model_uri=INFORES.agent_type, domain=None, range=Optional[Union[str, "AgentTypeEnum"]])



import os
from urllib3.util.ssl_ import create_urllib3_context
import yaml

ctx = create_urllib3_context()
ctx.load_default_certs()
ctx.options |= 0x4  # ssl.OP_LEGACY_SERVER_CONNECT

INFORES_YAML = os.path.join('../../../infores_catalog.yaml')


def extract_github_sources():
    with open(INFORES_YAML, 'r') as yaml_file, open('github_sources.txt', 'w') as github_file, open('no_url_sources.txt', 'w') as no_url_file, open('wiki_links.txt', 'w') as wiki_links:
        data = yaml.safe_load(yaml_file)
        for infores in data.get('information_resources'):
            if "xref" in infores.keys():
                if not infores.get("deprecated"):
                    for xref in infores.get("xref"):
                        if xref.startswith("https://github.com") and not xref.startswith("https://github.com/NCATSTranslator/Translator-All/wiki"):
                            github_file.write(f"{infores.get('id')},{infores.get('name')},{xref}\n")
                        elif xref.startswith("https://github.com/NCATSTranslator/Translator-All/wiki"):
                            wiki_links.write(f"{infores.get('id')},{infores.get('name')},{xref}\n")
            else:
                if not infores.get("deprecated"):
                    no_url_file.write(f"{infores.get('id')}, {infores.get('name')}\n")


if __name__ == "__main__":
    extract_github_sources()



from __future__ import annotations
from datetime import datetime, date
from enum import Enum
from typing import List, Dict, Optional, Any, Union
from pydantic import BaseModel as BaseModel, ConfigDict, Field
import sys
if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal


metamodel_version = "None"
version = "1.0.0"

class WeakRefShimBaseModel(BaseModel):
   __slots__ = '__weakref__'

class ConfiguredBaseModel(WeakRefShimBaseModel,
                validate_assignment = True,
                validate_all = True,
                underscore_attrs_are_private = True,
                extra = 'forbid',
                arbitrary_types_allowed = True,
                use_enum_values = True):
    pass


class InformationResourceStatusEnum(str, Enum):
    """
    The status of the infores identifier
    """
    
    released = "released"
    
    deprecated = "deprecated"
    
    draft = "draft"
    
    modified = "modified"
    
    

class KnowledgeLevelEnum(str, Enum):
    """
    The level of knowledge that supports an edge or node.  This is a general categorization of the type of evidence that supports a statement, and is not intended to be a comprehensive description of the evidence.  For example, a statement may be supported by a single publication, but that publication may contain multiple types of evidence, such as a computational prediction and a manual curation.  In this case, the knowledge level would be "curated", and the evidence would be described in more detail in the evidence graph.
    """
    # knowledge generated through manual curation  or interpretation of data or published study results
    curated = "curated"
    # predictions generated computationally through inference over less direct forms of evidence (without human  intervention or review)
    predicted = "predicted"
    # knowledge extracted from published text by NLP agents (without human intervention or review)
    text_mined = "text_mined"
    # statistical correlations calculated between variables in a clinical or omics dataset, by an automated  analysis pipeline
    correlation = "correlation"
    # edge reports a phenomenon that was reported/observed to have occurred (and possibly some quantification,  e.g. how many times, at what frequency)
    observed = "observed"
    # knowledge level may not fit into the categories above, or is not provided/known
    other = "other"
    # used for sources that might provide edges with different knowledge levels, e.g.correlations in addition to  curated Edges - set tag to Curated, unless predicate rules override
    mixed = "mixed"
    
    

class AgentTypeEnum(str, Enum):
    """
    The type of agent that supports an edge or node.  This is a general categorization of the type of agent that supports a statement, and is not intended to be a comprehensive description of the agent.  For example, a statement may be supported by a single publication, but that publication may contain multiple types of evidence, such as a computational prediction and a manual curation.  In this case, the agent type would be "publication", and the evidence would be described in more detail in the evidence graph.
    """
    # agent type is not provided or known
    not_provided = "not_provided"
    # a computational model, such as a machine learning model
    computational_model = "computational_model"
    
    

class InformationResourceContainer(ConfiguredBaseModel):
    """
    A collection of information resources
    """
    information_resources: Optional[List[InformationResource]] = Field(default_factory=list, description="""a collection of information resources""")
    

class InformationResource(ConfiguredBaseModel):
    """
    A database or knowledgebase and its supporting ecosystem of interfaces  and services that deliver content to consumers (e.g. web portals, APIs,  query endpoints, streaming services, data downloads, etc.). A single Information Resource by this definition may span many different datasets or databases, and include many access endpoints and user interfaces. Information Resources include project-specific resources such as a Translator Knowledge Provider, and community knowledgebases like ChemBL, OMIM, or DGIdb.
    """
    status: Optional[InformationResourceStatusEnum] = Field(None, description="""the status of the infores identifier, the default is \"released\"""")
    name: Optional[str] = Field(None, description="""A human-readable name for an attribute or entity.""")
    id: str = Field(..., description="""A unique identifier for an entity. Must be either a CURIE shorthand for a URI or a complete URI""")
    xref: Optional[List[str]] = Field(default_factory=list, description="""A database cross reference or alternative identifier for a NamedThing or edge between two  NamedThings.  This property should point to a database record or webpage that supports the existence of the edge, or  gives more detail about the edge. This property can be used on a node or edge to provide multiple URIs or CURIE cross references.""")
    synonym: Optional[List[str]] = Field(default_factory=list, description="""Alternate human-readable names for a thing""")
    description: Optional[str] = Field(None, description="""A free-text description of an entity or attribute.""")
    knowledge_level: Optional[KnowledgeLevelEnum] = Field(None, description="""The level of knowledge that supports an edge or node.  This is a general categorization of the type of evidence that supports a statement, and is not intended to be a comprehensive description of the evidence.  For example, a statement may be supported by a single publication, but that publication may contain multiple types of evidence, such as a computational prediction and a manual curation.  In this case, the knowledge level would be \"curated\", and the evidence would be described in more detail in the evidence graph.""")
    agent_type: Optional[AgentTypeEnum] = Field(None, description="""The type of agent that supports an edge or node.  This is a general categorization of the type of agent that supports a statement, and is not intended to be a comprehensive description of the agent.  For example, a statement may be supported by a single publication, but that publication may contain multiple types of evidence, such as a computational prediction and a manual curation.  In this case, the agent type would be \"publication\", and the evidence would be described in more detail in the evidence graph.""")
    


# Update forward refs
# see https://pydantic-docs.helpmanual.io/usage/postponed_annotations/
InformationResourceContainer.update_forward_refs()
InformationResource.update_forward_refs()




import os
import requests
import csv
import time
import urllib3
from urllib3.util.ssl_ import create_urllib3_context
import yaml

ctx = create_urllib3_context()
ctx.load_default_certs()
ctx.options |= 0x4  # ssl.OP_LEGACY_SERVER_CONNECT

INFORES_YAML = os.path.join('infores_catalog.yaml')


def is_valid_urls(url: str) -> bool:
    retries = 3
    url = url[0]
    print("Checking URL: " + url)
    for i in range(retries):
        try:
            with urllib3.PoolManager(ssl_context=ctx) as http:
                response = http.request("GET", url, headers={'User-Agent': 'Mozilla/5.0'})
                if response.status == 200:
                    return True
                else:
                    print(f"Response status code: {response.status}. Retrying...")
                    time.sleep(1)
        except requests.exceptions.RequestException as e:
            print(url)
    try:
        resp = requests.get(url)
        if resp.status_code == 200:
            return True
    except requests.exceptions.RequestException as e:
        print(f"Exception: {e}")
        print(f"Response status code: {response.status}.", url)
    return False


class InformationResource:

    def __init__(self) -> None:
        infores_map = {}

    def dump(self):
        raise NotImplementedError

    def validate(self):
        with open(INFORES_YAML, 'r') as yaml_file:
            data = yaml.safe_load(yaml_file)
            for infores in data.get('information_resources'):
                # exceptions for resolvable URLs that don't return 200 response for some reason (e.g. require
                # user to accept a popup before resolving):
                if infores.get("status") == "deprecated":
                    continue
                if infores.get("status") not in ["released", "deprecated", "draft", "modified"]:
                    print(infores)
                    print("Invalid infores status:" + infores.get("status")
                          + " for " + infores.get("name"))
                    raise ValueError("invalid status for " + infores.get("name") + " for " + infores.get("id"))
                if infores.get("knowledge level") not in ["curated",
                                                          "predicted",
                                                          "text_mined",
                                                          "correlation",
                                                          "observed",
                                                          "other",
                                                          "mixed"]:
                    print(infores)
                    print("Invalid infores knowledge level:" + infores.get("knowledge level")
                          + " for " + infores.get("name"))
                    raise ValueError("invalid knowledge level for " + infores.get("name") + " for " + infores.get("id"))

                if infores.get("agent type") not in ["not_provided", "computational_model"]:
                    print(infores)
                    print("Invalid infores agent type:" + infores.get("agent type") + " for " + infores.get("name"))
                    raise ValueError("invalid agent type for " + infores.get("name") + " for " + infores.get("id"))

                if infores.get("id") == 'infores:athena' \
                        or infores.get("id") == 'infores:isb-wellness' \
                        or infores.get("id") == 'infores:isb-incov' \
                        or infores.get("id") == 'infores:preppi' \
                        or infores.get("id") == 'infores:ttd' \
                        or infores.get("id") == 'infores:flybase' \
                        or infores.get("id") == 'infores:xenbase' \
                        or infores.get("id") == 'infores:aeolus' \
                        or infores.get("id") == 'infores:ctrp' \
                        or infores.get("id") == 'infores:date' \
                        or infores.get("id") == "infores:mirbase" \
                        or infores.get("id") == "infores:nsides" \
                        or infores.get("xref") is None \
                        or infores.get("status") == 'deprecated' \
                        or is_valid_urls(infores.get("xref")):
                    print(infores.get('id'), "has valid URL (xref)")
                else:
                    print(infores)
                    print("Invalid infores URL:" + " for " + infores.get("name"))
                    raise ValueError("invalid URL" + infores.get("name") + " for " + infores.get("id"))


if __name__ == "__main__":
    InformationResource().validate()
